package processor

import (
	"context"
	"fmt"

	"github.com/mailchain/mailchain/cmd/indexer/internal/actions"
	"github.com/mailchain/mailchain/cmd/indexer/internal/clients"
	"github.com/mailchain/mailchain/cmd/internal/datastore"
)

type Sequential struct {
	protocol string
	network  string

	syncStore      datastore.SyncStore
	blockProcessor actions.Block
	blockClient    clients.Block
}

func NewSequential(protocol, network string, store datastore.SyncStore, proc actions.Block, client clients.Block) *Sequential {
	return &Sequential{
		protocol:       protocol,
		network:        network,
		syncStore:      store,
		blockProcessor: proc,
		blockClient:    client,
	}
}

func (s *Sequential) NextBlock(ctx context.Context) error {
	blkNo, err := s.syncStore.GetBlockNumber(ctx, s.protocol, s.network)
	if err != nil {
		return err
	}

	fmt.Println("block number: ", blkNo)

	blk, err := s.blockClient.BlockByNumber(ctx, blkNo)
	if err != nil {
		return err
	}

	if err := s.blockProcessor.Run(ctx, s.protocol, s.network, blk); err != nil {
		return err
	}

	nextBlockNo := blkNo + 1

	fmt.Println("next block number: ", nextBlockNo)

	if err := s.syncStore.PutBlockNumber(ctx, s.protocol, s.network, nextBlockNo); err != nil {
		return err
	}

	return nil
}
